@@ -24,13 +24,14 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwidget.h>
+#include <gdk/gdkx.h>
 #include <bonobo/bonobo-context.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-generic-factory.h>
 #include <bonobo/bonobo-main.h>
-#include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -41,13 +42,15 @@
 #include "GNOME_Yelp.h"
 #include "yelp-window.h"
 #include "yelp-base.h"
+#include "yelp-html.h"
 
 #define YELP_FACTORY_OAFIID "OAFIID:GNOME_Yelp_Factory"
 
-poptContext  poptCon;
-gint         next_opt;
-gchar       *cache_dir;
-gchar       *open_urls;
+static poptContext  poptCon;
+/* static gint         next_opt; */
+static gchar       *cache_dir;
+static gchar       *open_urls;
+static gchar       *startup_id;
 
 /*structure defining command line option.*/
 enum {
@@ -75,8 +78,10 @@ static void           main_client_die         (GnomeClient          *client,
 					       gpointer              cdata);
 
 static gboolean	      main_restore_session    (void);
+static Time slowly_and_stupidly_obtain_timestamp (Display *xdisplay);
 
-struct poptOption options[] = {
+
+static struct poptOption options[] = {
 	{
 		"open-urls",
 		'\0',
@@ -114,7 +119,7 @@ main_base_factory (BonoboGenericFactory *factory,
 }
 
 static CORBA_Object
-main_activate_base ()
+main_activate_base (void)
 {
 	CORBA_Environment ev;
 	CORBA_Object      yelp_base;
@@ -141,12 +146,13 @@ main_open_new_window (CORBA_Object yelp_base, const gchar *url)
 	
 	CORBA_exception_init (&ev);
 
-	GNOME_Yelp_newWindow (yelp_base, url, &ev);
+	GNOME_Yelp_newWindow (yelp_base, url, startup_id, &ev);
 
 	if (BONOBO_EX (&ev)) {
 		g_error (_("Could not open new window."));
 	}
 
+	g_free (startup_id);
 	CORBA_exception_free (&ev);
 }
 	
@@ -154,16 +160,20 @@ static void
 main_start (gchar *url) 
 {
 	CORBA_Object yelp_base;
-	
+	gchar *new_url;
+
+	new_url = gnome_vfs_make_uri_from_input_with_dirs (url, 
+				GNOME_VFS_MAKE_URI_DIR_CURRENT);
+
 	yelp_base = main_activate_base ();
 
 	if (!yelp_base) {
 		g_error ("Couldn't activate YelpBase");
 	}
-	
-	main_open_new_window (yelp_base, url);
+
+	main_open_new_window (yelp_base, new_url);
 	gdk_notify_startup_complete ();
-	
+
 	bonobo_object_release_unref (yelp_base, NULL);
 
 	if (url) {
@@ -302,6 +312,57 @@ main_restore_session (void)
 	return FALSE;
 }
 
+/* Copied from libnautilus/nautilus-program-choosing.c; Needed in case
+ * we have no DESKTOP_STARTUP_ID (with its accompanying timestamp).
+ */
+static Time
+slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
+{
+	Window xwindow;
+	XEvent event;
+	
+	{
+		XSetWindowAttributes attrs;
+		Atom atom_name;
+		Atom atom_type;
+		gchar* name;
+		
+		attrs.override_redirect = True;
+		attrs.event_mask = PropertyChangeMask | StructureNotifyMask;
+		
+		xwindow =
+			XCreateWindow (xdisplay,
+				       RootWindow (xdisplay, 0),
+				       -100, -100, 1, 1,
+				       0,
+				       CopyFromParent,
+				       CopyFromParent,
+				       CopyFromParent,
+				       CWOverrideRedirect | CWEventMask,
+				       &attrs);
+		
+		atom_name = XInternAtom (xdisplay, "WM_NAME", TRUE);
+		g_assert (atom_name != None);
+		atom_type = XInternAtom (xdisplay, "STRING", TRUE);
+		g_assert (atom_type != None);
+		
+		name = "Fake Window";
+		XChangeProperty (xdisplay, 
+				 xwindow, atom_name,
+				 atom_type,
+				 8, PropModeReplace, name, strlen (name));
+	}
+	
+	XWindowEvent (xdisplay,
+		      xwindow,
+		      PropertyChangeMask,
+		      &event);
+	
+	XDestroyWindow(xdisplay, xwindow);
+	
+	return event.xproperty.time;
+}
+
 int
 main (int argc, char **argv) 
 {
@@ -311,17 +372,31 @@ main (int argc, char **argv)
 	GnomeClient   *client;
 	gboolean       session_started = FALSE;
 	const gchar  **args;
-	
-	
+	gchar *local_id;
+
 	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);  
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
+	local_id = (gchar *) g_getenv ("DESKTOP_STARTUP_ID");
+
+	if (local_id != NULL && *local_id != '\0') {
+		startup_id = g_strdup (local_id);
+		putenv ("DESKTOP_STARTUP_ID=");
+	}
+
+	gtk_window_set_auto_startup_notification(FALSE);
+
 	program = gnome_program_init (PACKAGE, VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PARAM_POPT_TABLE, options,
 				      GNOME_PROGRAM_STANDARD_PROPERTIES,
 				      NULL);
+	if (!startup_id) {
+		Time tmp;
+		tmp = slowly_and_stupidly_obtain_timestamp (gdk_display);
+		startup_id = g_strdup_printf ("_TIME%lu", tmp);
+	}
 	g_set_application_name (_("Help"));
 	gtk_window_set_default_icon_name ("gnome-help");
 
@@ -332,7 +407,9 @@ main (int argc, char **argv)
 		 gdk_display_get_name (gdk_display_get_default ()) );
 
 	gnome_vfs_init ();
-	
+
+	yelp_html_initialize ();
+
 	/* Commandline parsing is done here */
 	g_object_get (G_OBJECT (program),
 		      GNOME_PARAM_POPT_CONTEXT, &poptCon,
@@ -363,6 +440,17 @@ main (int argc, char **argv)
 	if (!factory) { /* Not started, start now */ 
 		BonoboGenericFactory *factory;
 		char                 *registration_id;
+		const gchar          *env;
+
+		/* workaround for bug #329461 */
+		env = g_getenv ("MOZ_ENABLE_PANGO");
+
+		if (env == NULL ||
+		    *env == '\0' ||
+		    g_str_equal(env, "0")) 
+		{
+			g_setenv ("MOZ_DISABLE_PANGO", "1", TRUE);
+		}
 
 		registration_id = bonobo_activation_make_registration_id (
 					YELP_FACTORY_OAFIID,
